<?php namespace EwTools\Handler;

use EwTools\Support\Date;
use EwTools\Support\Str;

/**
 * OpenContentQueryBuilder
 *
 * @link    http://infomaker.se
 * @package EwTools\Handler
 * @since   EwTools\Handler\OpenContentQueryBuilder 0.1
 */
class OpenContentQueryBuilder {
    
    /**
     * Contains the query to use against Open Content
     *
     * @var string
     */
    private $query_string;
    
    /**
     * Contains a string to be used as text-search and will therefor be escaped
     *
     * @var string
     */
    private $text;
    
    private function __construct( $query, $text = '' ) {
        $this->setQuery( $query );
        $this->setText( $text );
    }
    
    /**
     * Append string to query
     *
     * @param string $query
     *
     * @since 0.1
     * @return OpenContentQueryBuilder
     */
    public function append( $query ) {
        if( Str::notEmpty( $query ) ) {
            $this->setQuery( Str::append( trim( $query ), $this->getQuery(), ' ' ) );
        }
        
        return $this;
    }
    
    /**
     * Build and retrieve the finished query string
     *
     * @since 0.1
     * @return string
     */
    public function buildQueryString() {
        $text  = $this->getSanitizedText();
        $query = $this->getQuery();
        
        return Str::notEmpty( $text ) ? Str::prepend( "({$text}) AND", $query, ' ' ) : $query;
    }
    
    /**
     * Helper function for appending conditional query string
     *
     * @param string $query
     *
     * @since 0.1
     * @return OpenContentQueryBuilder
     */
    public function andIf( $query ) {
        return $this->append( " AND {$query}" );
    }
    
    /**
     * Helper function for appending conditional query string
     *
     * @param string       $property The name of the property to match with
     * @param string|array $values   One or a list of values to look for
     * @param string       $glue
     *
     * @return OpenContentQueryBuilder
     * @since 0.1
     */
    public function andIfProperty( $property, $values, $glue = 'OR' ) {
        return $this->andIf( static::propertySearch( $property, $values, $glue ) );
    }
    
    /**
     * Helper function for appending conditional query string
     *
     * @param string $query
     *
     * @since 0.1
     * @return OpenContentQueryBuilder
     */
    public function andIfNot( $query ) {
        return $this->append( " AND NOT {$query}" );
    }
    
    /**
     * Helper function for appending conditional query string
     *
     * @param string       $property The name of the property to match with
     * @param string|array $values   One or a list of values to look for
     * @param string       $glue
     *
     * @return OpenContentQueryBuilder
     * @since 0.1
     */
    public function andIfNotProperty( $property, $values, $glue = 'OR' ) {
        return $this->andIfNot( static::propertySearch( $property, $values, $glue ) );
    }
    
    /**
     * Helper function for appending conditional query string
     *
     * @param string $query
     *
     * @since 0.1
     * @return OpenContentQueryBuilder
     */
    public function orIf( $query ) {
        return $this->append( " OR {$query}" );
    }
    
    /**
     * Helper function for appending conditional query string
     *
     * @param string       $property The name of the property to match with
     * @param string|array $values   One or a list of values to look for
     * @param string       $glue
     *
     * @return OpenContentQueryBuilder
     * @since 0.1
     */
    public function orIfProperty( $property, $values, $glue = 'OR' ) {
        return $this->orIf( static::propertySearch( $property, $values, $glue ) );
    }
    
    /**
     * Helper function for appending conditional query string
     *
     * @param string $query
     *
     * @since 0.1
     * @return OpenContentQueryBuilder
     */
    public function orIfNot( $query ) {
        return $this->append( " OR NOT {$query}" );
    }
    
    /**
     * Helper function for appending conditional query string
     *
     * @param string       $property The name of the property to match with
     * @param string|array $values   One or a list of values to look for
     * @param string       $glue
     *
     * @return OpenContentQueryBuilder
     * @since 0.1
     */
    public function orIfNotProperty( $property, $values, $glue = 'OR' ) {
        return $this->orIfNot( static::propertySearch( $property, $values, $glue ) );
    }
    
    /**
     * Helper function to determine if the builder will generate an empty string
     *
     * @since 0.1
     * @return bool
     */
    public function isEmpty() {
        return Str::isEmpty( $this->buildQueryString() );
    }
    
    /**
     * Getter for raw query string
     *
     * @since 0.1
     * @return string
     */
    public function getQuery() {
        return $this->query_string;
    }
    
    /**
     * Getter for raw text search
     *
     * @since 0.1
     * @return string
     */
    public function getText() {
        return $this->text;
    }
    
    /**
     * Retrieve a sanitize version of the text where we escape characters that might make a query
     *
     * @since 0.1
     * @return string
     */
    public function getSanitizedText() {
        $text = stripcslashes( $this->getText() );
        
        return Str::notEmpty( $text ) ? '"' . addslashes( $text ) . '"' : $text;
    }
    
    /**
     * Prepend string to query
     *
     * @param string $query
     *
     * @since 0.1
     * @return OpenContentQueryBuilder
     */
    public function prepend( $query ) {
        if( Str::notEmpty( $query ) ) {
            $this->setQuery( Str::prepend( trim( $query ), $this->getQuery(), ' ' ) );
        }
        
        return $this;
    }
    
    /**
     * Setter for text search
     *
     * @param string $text
     *
     * @since 0.1
     * @return OpenContentQueryBuilder
     */
    public function setText( $text ) {
        $this->text = trim( $text );
        
        return $this;
    }
    
    /**
     * Setter for query
     *
     * @param string $query
     *
     * @since 0.1
     * @return OpenContentQueryBuilder
     */
    public function setQuery( $query ) {
        $this->query_string = trim( $query );
        
        return $this;
    }
    
    /**
     * The string version of this object is the built query
     *
     * @since 0.1
     * @return string
     */
    public function __toString() {
        return $this->buildQueryString();
    }
    
    /**
     * Create a Solr-property string
     *
     * @uses  Str::contains()
     *
     * @param string $name
     * @param string $value
     *
     * @since 0.1
     * @return string
     */
    public static function createProperty( $name, $value ) {
        $value = Str::contains( $value, ' ' ) ? '"' . $value . '"' : $value;
        
        return "{$name}:{$value}";
    }
    
    /**
     * Create a date readable by Open Content
     *
     * @param string $date
     *
     * @since 0.1
     * @return string
     */
    public static function createOcDate( $date = '' ) {
        return Date::parse( $date )->toOcString();
    }
    
    /**
     * Create a Solr-query block indicating a date-span
     *
     * @param string $property
     * @param string $from
     * @param string $to
     *
     * @return string
     * @since 0.1
     */
    public static function dateSpanSearch( $property, $from = '', $to = '' ) {
        $span_start = Str::notEmpty( $from ) ? static::createOcDate( $from ) : '*';
        $span_stop  = Str::notEmpty( $to ) ? static::createOcDate( $to ) : 'NOW';
        
        return "{$property}:[{$span_start} TO {$span_stop}]";
    }
    
    /**
     * Creates a Solr-query of one or more properties with the option to change the separator
     *
     * @param string       $property The name of the property to match with
     * @param string|array $values   One or a list of values to look for
     * @param string       $glue
     *
     * @since 0.1
     * @return string
     */
    public static function propertySearch( $property, $values, $glue = 'OR' ) {
        $values = array_unique( array_filter( (array)$values ) );
        
        if( ! empty( $values ) ) {
            $glue = trim( $glue ); // Make sure that there is no space around the glue
            
            $block = implode( " {$glue} ", array_map( function ( $value ) use ( $property ) {
                return static::createProperty( $property, $value );
            }, $values ) );
            
            return count( $values ) > 1 ? "({$block})" : $block;
        }
        
        return '';
    }
    
    /**
     * Start a query by property search
     *
     * @param string       $property The name of the property to match with
     * @param string|array $values   One or a list of values to look for
     * @param string       $glue
     *
     * @since 0.1
     * @return OpenContentQueryBuilder
     */
    public static function where( $property, $values, $glue = 'OR' ) {
        return static::query( static::propertySearch( $property, $values, $glue ) );
    }
    
    /**
     * Start a query
     *
     * @param string $query
     *
     * @since 0.1
     * @return OpenContentQueryBuilder
     */
    public static function query( $query ) {
        return new static( $query );
    }
    
    /**
     * Start a query by adding text
     *
     * @param string $text
     *
     * @since 0.1
     * @return OpenContentQueryBuilder
     */
    public static function text( $text ) {
        return new static( '', $text );
    }
}