<?php
/**
 * Project startup: Handles the basics of kickstarting everything needed for the Site to function properly
 */
namespace EwTools;

use EwTools\Support\Str;
use RuntimeException;

/**
 * ProjectStartup
 *
 * @link    http://infomaker.se
 * @package EwTools
 * @since   EwTools\ProjectStartup 0.1
 */
abstract class ProjectStartup {
    
    /**
     * Contains the path to the project-plugin with all project
     *
     * @var string
     */
    protected $plugin_path = '';
    
    /**
     * Contains the type of environment we are in.
     *
     * @use getenv('ENVIRONMENT')
     * @var string
     */
    protected $env;
    
    /**
     * Contains a list of Wordpress widgets to be unregistered since we don't use them
     *
     * @var array
     */
    protected $unregister_widgets = [
        'WP_Widget_Pages',
        'WP_Widget_Links',
        'WP_Widget_Search',
        'WP_Widget_Archives',
        'WP_Widget_Meta',
        'WP_Widget_Calendar',
        'WP_Widget_Categories',
        'WP_Widget_Recent_Posts',
        'WP_Widget_Recent_Comments',
        'WP_Widget_RSS',
        'WP_Widget_Tag_Cloud',
        'WP_Nav_Menu_Widget',
    ];
    
    /**
     *
     *
     * @var null
     */
    protected static $instance = null;
    
    /**
     * ProjectStartup constructor.
     *
     * @since 0.1
     *
     * @param string $plugin_path
     */
    private function __construct( $plugin_path ) {
        $this->plugin_path = $plugin_path;
        $this->env         = getenv( 'ENVIRONMENT' );
        
        $this->setupConstants()->unregisterWidgets()->removeWpEmojicons()->setupNotFoundRedirect();
        
        $this->setup();
        
    }
    
    /**
     * Project Setup
     *
     * @since 0.1
     * @return void
     */
    abstract protected function setup();
    
    /**
     * Function to check if we are running in dev environment
     *
     * @since 0.1
     * @return bool
     */
    protected function isDev() {
        $env = Str::lower( $this->env );
        
        return Str::not( 'prod', $env ) && Str::not( 'stage', $env );
    }
    
    /**
     * Unregister Widgets.
     * Remove WordPress standard widgets for cleaner GUI.
     *
     * @since 0.1
     * @return $this
     */
    private function unregisterWidgets() {
        
        add_action( 'widgets_init', function () {
            foreach ( $this->unregister_widgets as $widget ) {
                unregister_widget( $widget );
            }
        } );
        
        return $this;
    }
    
    /**
     * @since 0.1
     * @return void
     */
    protected function useToolsAdminStyle() {
        add_action( 'admin_init', function () {
            wp_enqueue_media();
        } );
    
        
        add_action( 'admin_enqueue_scripts', function() {
            if( is_admin() ) {
                $base_path = $this->getToolsAssetsPath();
                $script_version = $this->isDev()? false : GIT_COMMIT;
                
                wp_enqueue_style( 'ewtools-style', "{$base_path}/css/admin.min.css", [], $script_version, 'all' );
                wp_enqueue_script( 'ewtools-js', "{$base_path}/js/admin.min.js", ['jquery'], $script_version, true );
            }
        });
    }
    
    /**
     * Function to calculate the path to the tools assets
     *
     * @since 0.1
     * @return string
     */
    protected function getToolsAssetsPath() {
        $base_path = Str::startsWith(__DIR__, '/var/www') ? Str::substr(__DIR__ , Str::length('/var/www')) : __DIR__;
       
        return get_site_url(null, "{$base_path}/assets");
    }
    
    /**
     * Remove emojicons in WordPress
     *
     * @since 0.1
     * @return $this
     */
    private function removeWpEmojicons() {
        
        // All actions related to emojis
        remove_action( 'admin_print_styles', 'print_emoji_styles' );
        remove_action( 'wp_head', 'print_emoji_detection_script', 7 );
        remove_action( 'admin_print_scripts', 'print_emoji_detection_script' );
        remove_action( 'wp_print_styles', 'print_emoji_styles' );
        remove_filter( 'wp_mail', 'wp_staticize_emoji_for_email' );
        remove_filter( 'the_content_feed', 'wp_staticize_emoji' );
        remove_filter( 'comment_text_rss', 'wp_staticize_emoji' );
        
        return $this;
    }
    
    /**
     * Setup constants to be used allover the project
     *
     * @since 0.1
     * @return $this
     */
    private function setupConstants() {
        define( 'EWC_PLUGIN_BASE', $this->plugin_path );
        define( 'EVERY_DEVICE', isset( $_SERVER[ 'EVERY_DEVICE' ] ) ? $_SERVER[ 'EVERY_DEVICE' ] : 'desktop' );
        
        return $this;
    }
    
    /**
     * Don't let WP guess article urls for 404.
     *
     * @since 0.1
     * @return void
     */
    private function setupNotFoundRedirect() {
        add_filter( 'redirect_canonical', function ( $redirect_url ) {
            global $wp_query;
            
            if( is_404() ) {
                
                if( isset( $wp_query->query[ "p" ] ) && intval( $wp_query->query[ "p" ] ) > 0 && isset( $redirect_url ) && preg_match( '/.*\/article\/.*/', $redirect_url ) > 0 ) {
                    return $redirect_url;
                }
                
                return false;
            }
            
            return $redirect_url;
        } );
        
        return $this;
    }
    
    /**
     * Initialization for the project startup
     *
     * @param $plugin_path
     *
     * @since 0.1
     * @return static
     */
    public static function run( $plugin_path ) {
        
        if ( is_null(static::$instance) ) {
            static::$instance = new static( $plugin_path );
        }
        
        return static::$instance;
    }
    
    /**
     * Function to retrieve un instance of the startup Class
     *
     * @since 0.1
     * @throws RuntimeException
     * @return static
     */
    public static function getInstance() {
        if ( is_null(static::$instance) ) {
            throw new RuntimeException( get_called_class().' has not been instantiated yet! try '.get_called_class().'::run(__FILE__) from the bootstrap file of your project plugin.' );
        }
        return static::$instance;
    }
}